@@ -23,11 +23,12 @@ import (
 	"reflect"
 	"slices"
 
+	"golang.org/x/tools/go/ast/inspector"
 	"golang.org/x/tools/gopls/internal/label"
 	"golang.org/x/tools/gopls/internal/protocol"
 	"golang.org/x/tools/gopls/internal/util/astutil"
-	"golang.org/x/tools/gopls/internal/util/bug"
 	"golang.org/x/tools/gopls/internal/util/safetoken"
+	internalastutil "golang.org/x/tools/internal/astutil"
 	"golang.org/x/tools/internal/diff"
 	"golang.org/x/tools/internal/event"
 )
@@ -48,7 +49,7 @@ const (
 // Parse parses a buffer of Go source, repairing the tree if necessary.
 //
 // The provided ctx is used only for logging.
-func Parse(ctx context.Context, fset *token.FileSet, uri protocol.DocumentURI, src []byte, mode parser.Mode, purgeFuncBodies bool) (res *File, fixes []fixType) {
+func Parse(ctx context.Context, fset *token.FileSet, uri protocol.DocumentURI, src []byte, mode parser.Mode, purgeFuncBodies bool) (res *File, fixes []FixType) {
 	if purgeFuncBodies {
 		src = astutil.PurgeFuncBodies(src)
 	}
@@ -63,39 +64,8 @@ func Parse(ctx context.Context, fset *token.FileSet, uri protocol.DocumentURI, s
 	}
 	// Inv: file != nil.
 
-	// Workaround for #70162 (missing File{Start,End} when
-	// parsing empty file) with go1.23.
-	//
-	// When parsing an empty file, or one without a valid
-	// package declaration, the go1.23 parser bails out before
-	// setting FileStart and End.
-	//
-	// This leaves us no way to find the original
-	// token.File that ParseFile created, so as a
-	// workaround, we recreate the token.File, and
-	// populate the FileStart and FileEnd fields.
-	//
-	// See also #53202.
 	tokenFile := func(file *ast.File) *token.File {
-		tok := fset.File(file.FileStart)
-		if tok == nil {
-			// Invalid File.FileStart (also File.{Package,Name.Pos}).
-			if file.Package.IsValid() {
-				bug.Report("ast.File has valid Package but no FileStart")
-			}
-			if file.Name.Pos().IsValid() {
-				bug.Report("ast.File has valid Name.Pos but no FileStart")
-			}
-			tok = fset.AddFile(uri.Path(), -1, len(src))
-			tok.SetLinesForContent(src)
-			// If the File contained any valid token.Pos values,
-			// they would all be invalid wrt the new token.File,
-			// but we have established that it lacks FileStart,
-			// Package, and Name.Pos.
-			file.FileStart = token.Pos(tok.Base())
-			file.FileEnd = token.Pos(tok.Base() + tok.Size())
-		}
-		return tok
+		return fset.File(file.FileStart)
 	}
 
 	tok := tokenFile(file)
@@ -111,8 +81,8 @@ func Parse(ctx context.Context, fset *token.FileSet, uri protocol.DocumentURI, s
 			fixes = append(fixes, astFixes...)
 		}
 
-		for i := 0; i < 10; i++ {
-			// Fix certain syntax errors that render the file unparseable.
+		for i := range 10 {
+			// Fix certain syntax errors that render the file unparsable.
 			newSrc, srcFix := fixSrc(file, tok, src)
 			if newSrc == nil {
 				break
@@ -153,6 +123,11 @@ func Parse(ctx context.Context, fset *token.FileSet, uri protocol.DocumentURI, s
 	}
 	assert(file != nil, "nil *ast.File")
 
+	// Provide a cursor for fast and convenient navigation.
+	inspect := inspector.New([]*ast.File{file})
+	curFile, _ := inspect.Root().FirstChild()
+	_ = curFile.Node().(*ast.File)
+
 	return &File{
 		URI:      uri,
 		Mode:     mode,
@@ -161,6 +136,7 @@ func Parse(ctx context.Context, fset *token.FileSet, uri protocol.DocumentURI, s
 		fixedAST: fixedAST,
 		File:     file,
 		Tok:      tok,
+		Cursor:   curFile,
 		Mapper:   protocol.NewMapper(uri, src),
 		ParseErr: parseErr,
 	}, fixes
@@ -171,13 +147,18 @@ func Parse(ctx context.Context, fset *token.FileSet, uri protocol.DocumentURI, s
 //
 // If fixAST returns true, the resulting AST is considered "fixed", meaning
 // positions have been mangled, and type checker errors may not make sense.
-func fixAST(n ast.Node, tok *token.File, src []byte) (fixes []fixType) {
+func fixAST(n ast.Node, tok *token.File, src []byte) (fixes []FixType) {
 	var err error
-	walkASTWithParent(n, func(n, parent ast.Node) bool {
+	internalastutil.PreorderStack(n, nil, func(n ast.Node, stack []ast.Node) bool {
+		var parent ast.Node
+		if len(stack) > 0 {
+			parent = stack[len(stack)-1]
+		}
+
 		switch n := n.(type) {
 		case *ast.BadStmt:
 			if fixDeferOrGoStmt(n, parent, tok, src) {
-				fixes = append(fixes, fixedDeferOrGo)
+				fixes = append(fixes, FixedDeferOrGo)
 				// Recursively fix in our fixed node.
 				moreFixes := fixAST(parent, tok, src)
 				fixes = append(fixes, moreFixes...)
@@ -187,7 +168,7 @@ func fixAST(n ast.Node, tok *token.File, src []byte) (fixes []fixType) {
 			return false
 		case *ast.BadExpr:
 			if fixArrayType(n, parent, tok, src) {
-				fixes = append(fixes, fixedArrayType)
+				fixes = append(fixes, FixedArrayType)
 				// Recursively fix in our fixed node.
 				moreFixes := fixAST(parent, tok, src)
 				fixes = append(fixes, moreFixes...)
@@ -201,7 +182,7 @@ func fixAST(n ast.Node, tok *token.File, src []byte) (fixes []fixType) {
 			//   for i := foo
 			//
 			if fixInitStmt(n, parent, tok, src) {
-				fixes = append(fixes, fixedInit)
+				fixes = append(fixes, FixedInit)
 			}
 			return false
 		case *ast.SelectorExpr:
@@ -210,7 +191,7 @@ func fixAST(n ast.Node, tok *token.File, src []byte) (fixes []fixType) {
 			//   foo.var<> // want to complete to "foo.variance"
 			//
 			if fixPhantomSelector(n, tok, src) {
-				fixes = append(fixes, fixedPhantomSelector)
+				fixes = append(fixes, FixedPhantomSelector)
 			}
 			return true
 
@@ -220,7 +201,7 @@ func fixAST(n ast.Node, tok *token.File, src []byte) (fixes []fixType) {
 				// Adjust closing curly brace of empty switch/select
 				// statements so we can complete inside them.
 				if fixEmptySwitch(n, tok, src) {
-					fixes = append(fixes, fixedEmptySwitch)
+					fixes = append(fixes, FixedEmptySwitch)
 				}
 			}
 
@@ -232,66 +213,42 @@ func fixAST(n ast.Node, tok *token.File, src []byte) (fixes []fixType) {
 	return fixes
 }
 
-// walkASTWithParent walks the AST rooted at n. The semantics are
-// similar to ast.Inspect except it does not call f(nil).
-func walkASTWithParent(n ast.Node, f func(n ast.Node, parent ast.Node) bool) {
-	var ancestors []ast.Node
-	ast.Inspect(n, func(n ast.Node) (recurse bool) {
-		defer func() {
-			if recurse {
-				ancestors = append(ancestors, n)
-			}
-		}()
-
-		if n == nil {
-			ancestors = ancestors[:len(ancestors)-1]
-			return false
-		}
-
-		var parent ast.Node
-		if len(ancestors) > 0 {
-			parent = ancestors[len(ancestors)-1]
-		}
-
-		return f(n, parent)
-	})
-}
-
-// TODO(rfindley): revert this intrumentation once we're certain the crash in
+// TODO(rfindley): revert this instrumentation once we're certain the crash in
 // #59097 is fixed.
-type fixType int
+type FixType int
 
 const (
-	noFix fixType = iota
-	fixedCurlies
-	fixedDanglingSelector
-	fixedDeferOrGo
-	fixedArrayType
-	fixedInit
-	fixedPhantomSelector
-	fixedEmptySwitch
+	noFix FixType = iota
+	FixedCurlies
+	FixedDanglingSelector
+	FixedDeferOrGo
+	FixedArrayType
+	FixedInit
+	FixedPhantomSelector
+	FixedEmptySwitch
 )
 
 // fixSrc attempts to modify the file's source code to fix certain
 // syntax errors that leave the rest of the file unparsed.
 //
 // fixSrc returns a non-nil result if and only if a fix was applied.
-func fixSrc(f *ast.File, tf *token.File, src []byte) (newSrc []byte, fix fixType) {
-	walkASTWithParent(f, func(n, parent ast.Node) bool {
+func fixSrc(f *ast.File, tf *token.File, src []byte) (newSrc []byte, fix FixType) {
+	internalastutil.PreorderStack(f, nil, func(n ast.Node, stack []ast.Node) bool {
 		if newSrc != nil {
 			return false
 		}
 
 		switch n := n.(type) {
 		case *ast.BlockStmt:
+			parent := stack[len(stack)-1]
 			newSrc = fixMissingCurlies(f, n, parent, tf, src)
 			if newSrc != nil {
-				fix = fixedCurlies
+				fix = FixedCurlies
 			}
 		case *ast.SelectorExpr:
 			newSrc = fixDanglingSelector(n, tf, src)
 			if newSrc != nil {
-				fix = fixedDanglingSelector
+				fix = FixedDanglingSelector
 			}
 		}
 
@@ -446,8 +403,10 @@ func fixEmptySwitch(body *ast.BlockStmt, tok *token.File, src []byte) bool {
 	return true
 }
 
-// fixDanglingSelector inserts real "_" selector expressions in place
-// of phantom "_" selectors. For example:
+// fixDanglingSelector inserts a real "_" selector expression in place
+// of a phantom parser-inserted "_" selector so that the parser will
+// not consume the following non-identifier token.
+// For example:
 //
 //	func _() {
 //		x.<>
@@ -477,17 +436,13 @@ func fixDanglingSelector(s *ast.SelectorExpr, tf *token.File, src []byte) []byte
 		return nil
 	}
 
-	var buf bytes.Buffer
-	buf.Grow(len(src) + 1)
-	buf.Write(src[:insertOffset])
-	buf.WriteByte('_')
-	buf.Write(src[insertOffset:])
-	return buf.Bytes()
+	return slices.Concat(src[:insertOffset], []byte("_"), src[insertOffset:])
 }
 
-// fixPhantomSelector tries to fix selector expressions with phantom
-// "_" selectors. In particular, we check if the selector is a
-// keyword, and if so we swap in an *ast.Ident with the keyword text. For example:
+// fixPhantomSelector tries to fix selector expressions whose Sel is a
+// phantom (parser-invented) "_". If the text after the '.' is a
+// keyword, it updates Sel to a fake ast.Ident of that name. For
+// example:
 //
 // foo.var
 //
@@ -522,21 +477,18 @@ func fixPhantomSelector(sel *ast.SelectorExpr, tf *token.File, src []byte) bool
 	})
 }
 
-// isPhantomUnderscore reports whether the given ident is a phantom
-// underscore. The parser sometimes inserts phantom underscores when
-// it encounters otherwise unparseable situations.
+// isPhantomUnderscore reports whether the given ident from a
+// SelectorExpr.Sel was invented by the parser and is not present in
+// source text. The parser creates a blank "_" identifier when the
+// syntax (e.g. a selector) demands one but none is present. The fixer
+// also inserts them.
 func isPhantomUnderscore(id *ast.Ident, tok *token.File, src []byte) bool {
-	if id == nil || id.Name != "_" {
-		return false
+	switch id.Name {
+	case "_": // go1.24 parser
+		offset, err := safetoken.Offset(tok, id.Pos())
+		return err == nil && offset < len(src) && src[offset] != '_'
 	}
-
-	// Phantom underscore means the underscore is not actually in the
-	// program text.
-	offset, err := safetoken.Offset(tok, id.Pos())
-	if err != nil {
-		return false
-	}
-	return len(src) <= offset || src[offset] != '_'
+	return false // real
 }
 
 // fixInitStmt fixes cases where the parser misinterprets an
@@ -552,11 +504,12 @@ func fixInitStmt(bad *ast.BadExpr, parent ast.Node, tok *token.File, src []byte)
 	}
 
 	// Try to extract a statement from the BadExpr.
-	start, end, err := safetoken.Offsets(tok, bad.Pos(), bad.End()-1)
+	start, end, err := safetoken.Offsets(tok, bad.Pos(), bad.End())
 	if err != nil {
 		return false
 	}
-	stmtBytes := src[start : end+1]
+	assert(end <= len(src), "offset overflow") // golang/go#72026
+	stmtBytes := src[start:end]
 	stmt, err := parseStmt(tok, bad.Pos(), stmtBytes)
 	if err != nil {
 		return false
@@ -844,11 +797,7 @@ FindTo:
 // positions are valid.
 func parseStmt(tok *token.File, pos token.Pos, src []byte) (ast.Stmt, error) {
 	// Wrap our expression to make it a valid Go file we can pass to ParseFile.
-	fileSrc := bytes.Join([][]byte{
-		[]byte("package fake;func _(){"),
-		src,
-		[]byte("}"),
-	}, nil)
+	fileSrc := slices.Concat([]byte("package fake;func _(){"), src, []byte("}"))
 
 	// Use ParseFile instead of ParseExpr because ParseFile has
 	// best-effort behavior, whereas ParseExpr fails hard on any error.
@@ -896,8 +845,8 @@ var tokenPosType = reflect.TypeOf(token.NoPos)
 
 // offsetPositions applies an offset to the positions in an ast.Node.
 func offsetPositions(tok *token.File, n ast.Node, offset token.Pos) {
-	fileBase := int64(tok.Base())
-	fileEnd := fileBase + int64(tok.Size())
+	fileBase := token.Pos(tok.Base())
+	fileEnd := fileBase + token.Pos(tok.Size())
 	ast.Inspect(n, func(n ast.Node) bool {
 		if n == nil {
 			return false
@@ -917,23 +866,21 @@ func offsetPositions(tok *token.File, n ast.Node, offset token.Pos) {
 					continue
 				}
 
+				pos := token.Pos(f.Int())
+
 				// Don't offset invalid positions: they should stay invalid.
-				if !token.Pos(f.Int()).IsValid() {
+				if !pos.IsValid() {
 					continue
 				}
 
 				// Clamp value to valid range; see #64335.
 				//
 				// TODO(golang/go#64335): this is a hack, because our fixes should not
-				// produce positions that overflow (but they do: golang/go#64488).
-				pos := f.Int() + int64(offset)
-				if pos < fileBase {
-					pos = fileBase
-				}
-				if pos > fileEnd {
-					pos = fileEnd
-				}
-				f.SetInt(pos)
+				// produce positions that overflow (but they do; see golang/go#64488,
+				// #73438, #66790, #66683, #67704).
+				pos = min(max(pos+offset, fileBase), fileEnd)
+
+				f.SetInt(int64(pos))
 			}
 		}
 
@@ -976,7 +923,7 @@ func replaceNode(parent, oldChild, newChild ast.Node) bool {
 
 		switch f.Kind() {
 		// Check interface and pointer fields.
-		case reflect.Interface, reflect.Ptr:
+		case reflect.Interface, reflect.Pointer:
 			if tryReplace(f) {
 				return true
 			}
