@@ -38,13 +38,14 @@ import (
 	"golang.org/x/tools/gopls/internal/util/bug"
 	"golang.org/x/tools/gopls/internal/util/constraints"
 	"golang.org/x/tools/gopls/internal/util/immutable"
+	"golang.org/x/tools/gopls/internal/util/memoize"
+	"golang.org/x/tools/gopls/internal/util/moremaps"
 	"golang.org/x/tools/gopls/internal/util/pathutil"
 	"golang.org/x/tools/gopls/internal/util/persistent"
 	"golang.org/x/tools/gopls/internal/vulncheck"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/event/label"
 	"golang.org/x/tools/internal/gocommand"
-	"golang.org/x/tools/internal/memoize"
 )
 
 // A Snapshot represents the current state for a given view.
@@ -102,7 +103,7 @@ type Snapshot struct {
 	// initialErr holds the last error resulting from initialization. If
 	// initialization fails, we only retry when the workspace modules change,
 	// to avoid too many go/packages calls.
-	// If initialized is false, initialErr stil holds the error resulting from
+	// If initialized is false, initialErr still holds the error resulting from
 	// the previous initialization.
 	// TODO(rfindley): can we unify the lifecycle of initialized and initialErr.
 	initialErr *InitializationError
@@ -323,6 +324,8 @@ func fileKind(fh file.Handle) file.Kind {
 		return file.Sum
 	case ".work":
 		return file.Work
+	case ".s":
+		return file.Asm
 	}
 	return file.UnknownKind
 }
@@ -606,7 +609,7 @@ func (s *Snapshot) MethodSets(ctx context.Context, ids ...PackageID) ([]*methods
 	pre := func(i int, ph *packageHandle) bool {
 		data, err := filecache.Get(methodSetsKind, ph.key)
 		if err == nil { // hit
-			indexes[i] = methodsets.Decode(data)
+			indexes[i] = methodsets.Decode(ph.mp.PkgPath, data)
 			return false
 		} else if err != filecache.ErrNotFound {
 			event.Error(ctx, "reading methodsets from filecache", err)
@@ -645,6 +648,21 @@ func (s *Snapshot) Tests(ctx context.Context, ids ...PackageID) ([]*testfuncs.In
 	return indexes, s.forEachPackage(ctx, ids, pre, post)
 }
 
+// NarrowestMetadataForFile returns metadata for the narrowest package
+// (the one with the fewest files) that encloses the specified file.
+// The result may be a test variant, but never an intermediate test variant.
+func (snapshot *Snapshot) NarrowestMetadataForFile(ctx context.Context, uri protocol.DocumentURI) (*metadata.Package, error) {
+	mps, err := snapshot.MetadataForFile(ctx, uri)
+	if err != nil {
+		return nil, err
+	}
+	metadata.RemoveIntermediateTestVariants(&mps)
+	if len(mps) == 0 {
+		return nil, fmt.Errorf("no package metadata for file %s", uri)
+	}
+	return mps[0], nil
+}
+
 // MetadataForFile returns a new slice containing metadata for each
 // package containing the Go file identified by uri, ordered by the
 // number of CompiledGoFiles (i.e. "narrowest" to "widest" package),
@@ -652,6 +670,10 @@ func (s *Snapshot) Tests(ctx context.Context, ids ...PackageID) ([]*testfuncs.In
 // The result may include tests and intermediate test variants of
 // importable packages.
 // It returns an error if the context was cancelled.
+//
+// TODO(adonovan): in nearly all cases the caller must use
+// [metadata.RemoveIntermediateTestVariants]. Make this a parameter to
+// force the caller to consider it (and reduce code).
 func (s *Snapshot) MetadataForFile(ctx context.Context, uri protocol.DocumentURI) ([]*metadata.Package, error) {
 	if s.view.typ == AdHocView {
 		// As described in golang/go#57209, in ad-hoc workspaces (where we load ./
@@ -668,11 +690,11 @@ func (s *Snapshot) MetadataForFile(ctx context.Context, uri protocol.DocumentURI
 	s.mu.Lock()
 
 	// Start with the set of package associations derived from the last load.
-	ids := s.meta.IDs[uri]
+	pkgs := s.meta.ForFile[uri]
 
 	shouldLoad := false // whether any packages containing uri are marked 'shouldLoad'
-	for _, id := range ids {
-		if pkgs, _ := s.shouldLoad.Get(id); len(pkgs) > 0 {
+	for _, pkg := range pkgs {
+		if p, _ := s.shouldLoad.Get(pkg.ID); len(p) > 0 {
 			shouldLoad = true
 		}
 	}
@@ -686,7 +708,7 @@ func (s *Snapshot) MetadataForFile(ctx context.Context, uri protocol.DocumentURI
 	//  - uri is not contained in any valid packages
 	//  - ...or one of the packages containing uri is marked 'shouldLoad'
 	//  - ...but uri is not unloadable
-	if (shouldLoad || len(ids) == 0) && !unloadable {
+	if (shouldLoad || len(pkgs) == 0) && !unloadable {
 		scope := fileLoadScope(uri)
 		err := s.load(ctx, NoNetwork, scope)
 
@@ -719,18 +741,14 @@ func (s *Snapshot) MetadataForFile(ctx context.Context, uri protocol.DocumentURI
 	// Retrieve the metadata.
 	s.mu.Lock()
 	defer s.mu.Unlock()
-	ids = s.meta.IDs[uri]
-	metas := make([]*metadata.Package, len(ids))
-	for i, id := range ids {
-		metas[i] = s.meta.Packages[id]
-		if metas[i] == nil {
-			panic("nil metadata")
-		}
-	}
+	// TODO(rfindley): is there any reason not to make the sorting below the
+	// canonical sorting, so that we don't need to mutate this slice?
+	metas := slices.Clone(s.meta.ForFile[uri])
+
 	// Metadata is only ever added by loading,
 	// so if we get here and still have
-	// no IDs, uri is unloadable.
-	if !unloadable && len(ids) == 0 {
+	// no packages, uri is unloadable.
+	if !unloadable && len(metas) == 0 {
 		s.unloadableFiles.Add(uri)
 	}
 
@@ -774,10 +792,8 @@ func (s *Snapshot) ReverseDependencies(ctx context.Context, id PackageID, transi
 	} else {
 		// direct reverse dependencies
 		rdeps = make(map[PackageID]*metadata.Package)
-		for _, rdepID := range meta.ImportedBy[id] {
-			if rdep := meta.Packages[rdepID]; rdep != nil {
-				rdeps[rdepID] = rdep
-			}
+		for _, rdep := range meta.ImportedBy[id] {
+			rdeps[rdep.ID] = rdep
 		}
 	}
 
@@ -895,10 +911,12 @@ func (s *Snapshot) watchSubdirs() bool {
 		// requirements that client names do not change. We should update the VS
 		// Code extension to set a default value of "subdirWatchPatterns" to "on",
 		// so that this workaround is only temporary.
-		if s.Options().ClientInfo.Name == "Visual Studio Code" {
+		switch s.Options().ClientInfo.Name {
+		case "Visual Studio Code", "Visual Studio Code - Insiders":
 			return true
+		default:
+			return false
 		}
-		return false
 	default:
 		bug.Reportf("invalid subdirWatchPatterns: %q", p)
 		return false
@@ -944,9 +962,9 @@ func (s *Snapshot) WorkspaceMetadata(ctx context.Context) ([]*metadata.Package,
 	defer s.mu.Unlock()
 
 	meta := make([]*metadata.Package, 0, s.workspacePackages.Len())
-	s.workspacePackages.Range(func(id PackageID, _ PackagePath) {
+	for id, _ := range s.workspacePackages.All() {
 		meta = append(meta, s.meta.Packages[id])
-	})
+	}
 	return meta, nil
 }
 
@@ -978,19 +996,13 @@ func (s *Snapshot) IsWorkspacePackage(id PackageID) bool {
 // It may also contain ad-hoc packages for standalone files.
 // It includes all test variants.
 //
-// TODO(rfindley): Replace this with s.MetadataGraph().
+// TODO(rfindley): Replace usage of function this with s.LoadMetadataGraph().
 func (s *Snapshot) AllMetadata(ctx context.Context) ([]*metadata.Package, error) {
-	if err := s.awaitLoaded(ctx); err != nil {
+	g, err := s.LoadMetadataGraph(ctx)
+	if err != nil {
 		return nil, err
 	}
-
-	g := s.MetadataGraph()
-
-	meta := make([]*metadata.Package, 0, len(g.Packages))
-	for _, mp := range g.Packages {
-		meta = append(meta, mp)
-	}
-	return meta, nil
+	return moremaps.ValueSlice(g.Packages), nil
 }
 
 // GoModForFile returns the URI of the go.mod file for the given URI.
@@ -1043,9 +1055,8 @@ func (s *Snapshot) clearShouldLoad(scopes ...loadScope) {
 			}
 		case fileLoadScope:
 			uri := protocol.DocumentURI(scope)
-			ids := s.meta.IDs[uri]
-			for _, id := range ids {
-				s.shouldLoad.Delete(id)
+			for _, pkg := range s.meta.ForFile[uri] {
+				s.shouldLoad.Delete(pkg.ID)
 			}
 		}
 	}
@@ -1148,6 +1159,14 @@ func (s *Snapshot) MetadataGraph() *metadata.Graph {
 	return s.meta
 }
 
+// LoadMetadataGraph is like [Snapshot.MetadataGraph], but awaits snapshot reloading.
+func (s *Snapshot) LoadMetadataGraph(ctx context.Context) (*metadata.Graph, error) {
+	if err := s.awaitLoaded(ctx); err != nil {
+		return nil, err
+	}
+	return s.MetadataGraph(), nil
+}
+
 // InitializationError returns the last error from initialization.
 func (s *Snapshot) InitializationError() *InitializationError {
 	s.mu.Lock()
@@ -1301,7 +1320,7 @@ searchOverlays:
 			// where the file is inside a workspace module, but perhaps no packages
 			// were loaded for that module.
 			_, loadedMod := loadedModFiles[goMod]
-			_, workspaceMod := s.view.viewDefinition.workspaceModFiles[goMod]
+			_, workspaceMod := s.view.workspaceModFiles[goMod]
 			// If we have a relevant go.mod file, check whether the file is orphaned
 			// due to its go.mod file being inactive. We could also offer a
 			// prescriptive diagnostic in the case that there is no go.mod file, but
@@ -1398,7 +1417,7 @@ https://github.com/golang/tools/blob/master/gopls/doc/workspace.md.`, modDir, fi
 					fix = `This file may be excluded due to its build tags; try adding "-tags=<build tag>" to your gopls "buildFlags" configuration
 See the documentation for more information on working with build tags:
 https://github.com/golang/tools/blob/master/gopls/doc/settings.md#buildflags.`
-				} else if strings.Contains(filepath.Base(fh.URI().Path()), "_") {
+				} else if strings.Contains(fh.URI().Base(), "_") {
 					fix = `This file may be excluded due to its GOOS/GOARCH, or other build constraints.`
 				} else {
 					fix = `This file is ignored by your gopls build.` // we don't know why
@@ -1440,10 +1459,11 @@ func orphanedFileDiagnosticRange(ctx context.Context, cache *parseCache, fh file
 		return nil, protocol.Range{}, false
 	}
 	pgf := pgfs[0]
-	if !pgf.File.Name.Pos().IsValid() {
+	name := pgf.File.Name
+	if !name.Pos().IsValid() {
 		return nil, protocol.Range{}, false
 	}
-	rng, err := pgf.PosRange(pgf.File.Name.Pos(), pgf.File.Name.End())
+	rng, err := pgf.PosRange(name.Pos(), name.End())
 	if err != nil {
 		return nil, protocol.Range{}, false
 	}
@@ -1679,7 +1699,7 @@ func (s *Snapshot) clone(ctx, bgCtx context.Context, changed StateChange, done f
 		anyPkgFileChanged = anyPkgFileChanged || pkgFileChanged
 
 		// Mark all of the package IDs containing the given file.
-		filePackageIDs := invalidatedPackageIDs(uri, s.meta.IDs, pkgFileChanged)
+		filePackageIDs := invalidatedPackageIDs(uri, s.meta.ForFile, pkgFileChanged)
 		for id := range filePackageIDs {
 			directIDs[id] = directIDs[id] || invalidateMetadata // may insert 'false'
 		}
@@ -1738,7 +1758,7 @@ func (s *Snapshot) clone(ctx, bgCtx context.Context, changed StateChange, done f
 	//
 	// We could also do better by looking at which imports were deleted and
 	// trying to find cycles they are involved in. This fails when the file goes
-	// from an unparseable state to a parseable state, as we don't have a
+	// from an unparsable state to a parseable state, as we don't have a
 	// starting point to compare with.
 	if anyImportDeleted {
 		for id, mp := range s.meta.Packages {
@@ -1780,8 +1800,8 @@ func (s *Snapshot) clone(ctx, bgCtx context.Context, changed StateChange, done f
 			return
 		}
 		idsToInvalidate[id] = newInvalidateMetadata
-		for _, rid := range s.meta.ImportedBy[id] {
-			addRevDeps(rid, invalidateMetadata)
+		for _, rdep := range s.meta.ImportedBy[id] {
+			addRevDeps(rdep.ID, invalidateMetadata)
 		}
 	}
 	for id, invalidateMetadata := range directIDs {
@@ -1915,12 +1935,12 @@ func deleteMostRelevantModFile(m *persistent.Map[protocol.DocumentURI, *memoize.
 // If packageFileChanged is set, the file is either a new file, or has a new
 // package name. In this case, all known packages in the directory will be
 // invalidated.
-func invalidatedPackageIDs(uri protocol.DocumentURI, known map[protocol.DocumentURI][]PackageID, packageFileChanged bool) map[PackageID]struct{} {
+func invalidatedPackageIDs(uri protocol.DocumentURI, known map[protocol.DocumentURI][]*metadata.Package, packageFileChanged bool) map[PackageID]struct{} {
 	invalidated := make(map[PackageID]struct{})
 
 	// At a minimum, we invalidate packages known to contain uri.
-	for _, id := range known[uri] {
-		invalidated[id] = struct{}{}
+	for _, pkg := range known[uri] {
+		invalidated[pkg.ID] = struct{}{}
 	}
 
 	// If the file didn't move to a new package, we should only invalidate the
@@ -1953,15 +1973,15 @@ func invalidatedPackageIDs(uri protocol.DocumentURI, known map[protocol.Document
 	fi, err := getInfo(dir)
 	if err == nil {
 		// Aggregate all possibly relevant package IDs.
-		for knownURI, ids := range known {
+		for knownURI, pkgs := range known {
 			knownDir := knownURI.DirPath()
 			knownFI, err := getInfo(knownDir)
 			if err != nil {
 				continue
 			}
 			if os.SameFile(fi, knownFI) {
-				for _, id := range ids {
-					invalidated[id] = struct{}{}
+				for _, pkg := range pkgs {
+					invalidated[pkg.ID] = struct{}{}
 				}
 			}
 		}
@@ -2066,7 +2086,7 @@ func metadataChanges(ctx context.Context, lockedSnapshot *Snapshot, oldFH, newFH
 		} else {
 			// At this point, we shouldn't ever fail to produce a parsego.File, as
 			// we're already past header parsing.
-			bug.Reportf("metadataChanges: unparseable file %v (old error: %v, new error: %v)", oldFH.URI(), oldErr, newErr)
+			bug.Reportf("metadataChanges: unparsable file %v (old error: %v, new error: %v)", oldFH.URI(), oldErr, newErr)
 		}
 	}
 
