@@ -11,13 +11,13 @@ import (
 	"go/ast"
 	"go/token"
 	"go/types"
-	"path/filepath"
 	"reflect"
 	"slices"
-	"sort"
 	"strings"
 
 	"golang.org/x/tools/go/ast/astutil"
+	"golang.org/x/tools/go/ast/edge"
+	"golang.org/x/tools/go/ast/inspector"
 	"golang.org/x/tools/gopls/internal/analysis/fillstruct"
 	"golang.org/x/tools/gopls/internal/analysis/fillswitch"
 	"golang.org/x/tools/gopls/internal/cache"
@@ -41,8 +41,7 @@ import (
 //
 // See ../protocol/codeactionkind.go for some code action theory.
 func CodeActions(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle, rng protocol.Range, diagnostics []protocol.Diagnostic, enabled func(protocol.CodeActionKind) bool, trigger protocol.CodeActionTriggerKind) (actions []protocol.CodeAction, _ error) {
-
-	loc := protocol.Location{URI: fh.URI(), Range: rng}
+	loc := fh.URI().Location(rng)
 
 	pgf, err := snapshot.ParseGo(ctx, fh, parsego.Full)
 	if err != nil {
@@ -106,16 +105,17 @@ func CodeActions(ctx context.Context, snapshot *cache.Snapshot, fh file.Handle,
 			req.pkg = nil
 		}
 		if err := p.fn(ctx, req); err != nil {
-			// TODO(adonovan): most errors in code action providers should
-			// not block other providers; see https://go.dev/issue/71275.
-			return nil, err
+			// An error in one code action producer
+			// should not affect the others.
+			if ctx.Err() != nil {
+				return nil, err
+			}
+			event.Error(ctx, fmt.Sprintf("CodeAction producer %s failed", p.kind), err)
+			continue
 		}
 	}
 
-	sort.Slice(actions, func(i, j int) bool {
-		return actions[i].Kind < actions[j].Kind
-	})
-
+	// Return code actions in the order their providers are listed.
 	return actions, nil
 }
 
@@ -233,6 +233,8 @@ type codeActionProducer struct {
 	needPkg bool // fn needs type information (req.pkg)
 }
 
+// Code Actions are returned in the order their producers are listed below.
+// Depending on the client, this may influence the order they appear in the UI.
 var codeActionProducers = [...]codeActionProducer{
 	{kind: protocol.QuickFix, fn: quickFix, needPkg: true},
 	{kind: protocol.SourceOrganizeImports, fn: sourceOrganizeImports},
@@ -240,9 +242,9 @@ var codeActionProducers = [...]codeActionProducer{
 	{kind: settings.GoAssembly, fn: goAssembly, needPkg: true},
 	{kind: settings.GoDoc, fn: goDoc, needPkg: true},
 	{kind: settings.GoFreeSymbols, fn: goFreeSymbols},
-	{kind: settings.GoTest, fn: goTest},
+	{kind: settings.GoSplitPackage, fn: goSplitPackage, needPkg: true},
+	{kind: settings.GoTest, fn: goTest, needPkg: true},
 	{kind: settings.GoToggleCompilerOptDetails, fn: toggleCompilerOptDetails},
-	{kind: settings.GoplsDocFeatures, fn: goplsDocFeatures},
 	{kind: settings.RefactorExtractFunction, fn: refactorExtractFunction},
 	{kind: settings.RefactorExtractMethod, fn: refactorExtractMethod},
 	{kind: settings.RefactorExtractToNewFile, fn: refactorExtractToNewFile},
@@ -251,6 +253,7 @@ var codeActionProducers = [...]codeActionProducer{
 	{kind: settings.RefactorExtractConstantAll, fn: refactorExtractVariableAll, needPkg: true},
 	{kind: settings.RefactorExtractVariableAll, fn: refactorExtractVariableAll, needPkg: true},
 	{kind: settings.RefactorInlineCall, fn: refactorInlineCall, needPkg: true},
+	{kind: settings.RefactorInlineVariable, fn: refactorInlineVariable, needPkg: true},
 	{kind: settings.RefactorRewriteChangeQuote, fn: refactorRewriteChangeQuote},
 	{kind: settings.RefactorRewriteFillStruct, fn: refactorRewriteFillStruct, needPkg: true},
 	{kind: settings.RefactorRewriteFillSwitch, fn: refactorRewriteFillSwitch, needPkg: true},
@@ -260,6 +263,10 @@ var codeActionProducers = [...]codeActionProducer{
 	{kind: settings.RefactorRewriteMoveParamLeft, fn: refactorRewriteMoveParamLeft, needPkg: true},
 	{kind: settings.RefactorRewriteMoveParamRight, fn: refactorRewriteMoveParamRight, needPkg: true},
 	{kind: settings.RefactorRewriteSplitLines, fn: refactorRewriteSplitLines, needPkg: true},
+	{kind: settings.RefactorRewriteEliminateDotImport, fn: refactorRewriteEliminateDotImport, needPkg: true},
+	{kind: settings.RefactorRewriteAddTags, fn: refactorRewriteAddStructTags, needPkg: true},
+	{kind: settings.RefactorRewriteRemoveTags, fn: refactorRewriteRemoveStructTags, needPkg: true},
+	{kind: settings.GoplsDocFeatures, fn: goplsDocFeatures}, // offer this one last (#72742)
 
 	// Note: don't forget to update the allow-list in Server.CodeAction
 	// when adding new query operations like GoTest and GoDoc that
@@ -309,7 +316,7 @@ func quickFix(ctx context.Context, req *codeActionsRequest) error {
 	for _, typeError := range req.pkg.TypeErrors() {
 		// Does type error overlap with CodeAction range?
 		start, end := typeError.Pos, typeError.Pos
-		if _, _, endPos, ok := typesinternal.ReadGo116ErrorData(typeError); ok {
+		if _, _, endPos, ok := typesinternal.ErrorCodeStartEnd(typeError); ok {
 			end = endPos
 		}
 		typeErrorRange, err := req.pgf.PosRange(start, end)
@@ -325,8 +332,7 @@ func quickFix(ctx context.Context, req *codeActionsRequest) error {
 		case strings.Contains(msg, "missing method"),
 			strings.HasPrefix(msg, "cannot convert"),
 			strings.Contains(msg, "not implement"):
-			path, _ := astutil.PathEnclosingInterval(req.pgf.File, start, end)
-			si := stubmethods.GetIfaceStubInfo(req.pkg.FileSet(), info, path, start)
+			si := stubmethods.GetIfaceStubInfo(req.pkg.FileSet(), info, req.pgf, start, end)
 			if si != nil {
 				qual := typesinternal.FileQualifier(req.pgf.File, si.Concrete.Obj().Pkg())
 				iface := types.TypeString(si.Interface.Type(), qual)
@@ -338,8 +344,7 @@ func quickFix(ctx context.Context, req *codeActionsRequest) error {
 		// Offer a "Declare missing method T.f" code action.
 		// See [stubMissingCalledFunctionFixer] for command implementation.
 		case strings.Contains(msg, "has no field or method"):
-			path, _ := astutil.PathEnclosingInterval(req.pgf.File, start, end)
-			si := stubmethods.GetCallStubInfo(req.pkg.FileSet(), info, path, start)
+			si := stubmethods.GetCallStubInfo(req.pkg.FileSet(), info, req.pgf, start, end)
 			if si != nil {
 				msg := fmt.Sprintf("Declare missing method %s.%s", si.Receiver.Obj().Name(), si.MethodName)
 				req.addApplyFixAction(msg, fixMissingCalledFunction, req.loc)
@@ -436,6 +441,23 @@ func goFreeSymbols(ctx context.Context, req *codeActionsRequest) error {
 	return nil
 }
 
+// goSplitPackage produces "Split package p" code actions.
+// See [server.commandHandler.SplitPackage] for command implementation.
+func goSplitPackage(ctx context.Context, req *codeActionsRequest) error {
+	// TODO(adonovan): ideally we would key by the package path,
+	// or the ID of the widest package for the current file,
+	// so that we don't see different results when toggling
+	// between p.go and p_test.go.
+	//
+	// TODO(adonovan): opt: req should always provide metadata so
+	// that we don't have to request type checking (needPkg=true).
+	meta := req.pkg.Metadata()
+	title := fmt.Sprintf("Split package %q", meta.Name)
+	cmd := command.NewSplitPackageCommand(title, req.snapshot.View().ID(), string(meta.ID))
+	req.addCommandAction(cmd, false)
+	return nil
+}
+
 // goplsDocFeatures produces "Browse gopls feature documentation" code actions.
 // See [server.commandHandler.ClientOpenURL] for command implementation.
 func goplsDocFeatures(ctx context.Context, req *codeActionsRequest) error {
@@ -443,7 +465,7 @@ func goplsDocFeatures(ctx context.Context, req *codeActionsRequest) error {
 	// use the gopls release tag instead of master.
 	cmd := command.NewClientOpenURLCommand(
 		"Browse gopls feature documentation",
-		"https://github.com/golang/tools/blob/master/gopls/doc/features/README.md")
+		"https://github.com/golang/tools/blob/master/gopls/doc/features/index.md")
 	req.addCommandAction(cmd, false)
 	return nil
 }
@@ -462,7 +484,7 @@ func goDoc(ctx context.Context, req *codeActionsRequest) error {
 // refactorExtractFunction produces "Extract function" code actions.
 // See [extractFunction] for command implementation.
 func refactorExtractFunction(ctx context.Context, req *codeActionsRequest) error {
-	if _, ok, _, _ := canExtractFunction(req.pgf.Tok, req.start, req.end, req.pgf.Src, req.pgf.File); ok {
+	if _, ok, _, _ := canExtractFunction(req.pgf.Tok, req.start, req.end, req.pgf.Src, req.pgf.Cursor); ok {
 		req.addApplyFixAction("Extract function", fixExtractFunction, req.loc)
 	}
 	return nil
@@ -471,7 +493,7 @@ func refactorExtractFunction(ctx context.Context, req *codeActionsRequest) error
 // refactorExtractMethod produces "Extract method" code actions.
 // See [extractMethod] for command implementation.
 func refactorExtractMethod(ctx context.Context, req *codeActionsRequest) error {
-	if _, ok, methodOK, _ := canExtractFunction(req.pgf.Tok, req.start, req.end, req.pgf.Src, req.pgf.File); ok && methodOK {
+	if _, ok, methodOK, _ := canExtractFunction(req.pgf.Tok, req.start, req.end, req.pgf.Src, req.pgf.Cursor); ok && methodOK {
 		req.addApplyFixAction("Extract method", fixExtractMethod, req.loc)
 	}
 	return nil
@@ -481,7 +503,7 @@ func refactorExtractMethod(ctx context.Context, req *codeActionsRequest) error {
 // See [extractVariable] for command implementation.
 func refactorExtractVariable(ctx context.Context, req *codeActionsRequest) error {
 	info := req.pkg.TypesInfo()
-	if exprs, err := canExtractVariable(info, req.pgf.File, req.start, req.end, false); err == nil {
+	if exprs, err := canExtractVariable(info, req.pgf.Cursor, req.start, req.end, false); err == nil {
 		// Offer one of refactor.extract.{constant,variable}
 		// based on the constness of the expression; this is a
 		// limitation of the codeActionProducers mechanism.
@@ -502,17 +524,17 @@ func refactorExtractVariable(ctx context.Context, req *codeActionsRequest) error
 }
 
 // refactorExtractVariableAll produces "Extract N occurrences of EXPR" code action.
-// See [extractAllOccursOfExpr] for command implementation.
+// See [extractVariable] for implementation.
 func refactorExtractVariableAll(ctx context.Context, req *codeActionsRequest) error {
 	info := req.pkg.TypesInfo()
 	// Don't suggest if only one expr is found,
 	// otherwise it will duplicate with [refactorExtractVariable]
-	if exprs, err := canExtractVariable(info, req.pgf.File, req.start, req.end, true); err == nil && len(exprs) > 1 {
-		start, end, err := req.pgf.NodeOffsets(exprs[0])
+	if exprs, err := canExtractVariable(info, req.pgf.Cursor, req.start, req.end, true); err == nil && len(exprs) > 1 {
+		text, err := req.pgf.NodeText(exprs[0])
 		if err != nil {
 			return err
 		}
-		desc := string(req.pgf.Src[start:end])
+		desc := string(text)
 		if len(desc) >= 40 || strings.Contains(desc, "\n") {
 			desc = astutil.NodeDescription(exprs[0])
 		}
@@ -664,7 +686,7 @@ func refactorRewriteChangeQuote(ctx context.Context, req *codeActionsRequest) er
 // refactorRewriteInvertIf produces "Invert 'if' condition" code actions.
 // See [invertIfCondition] for command implementation.
 func refactorRewriteInvertIf(ctx context.Context, req *codeActionsRequest) error {
-	if _, ok, _ := canInvertIfCondition(req.pgf.File, req.start, req.end); ok {
+	if _, ok, _ := canInvertIfCondition(req.pgf.Cursor, req.start, req.end); ok {
 		req.addApplyFixAction("Invert 'if' condition", fixInvertIfCondition, req.loc)
 	}
 	return nil
@@ -674,17 +696,109 @@ func refactorRewriteInvertIf(ctx context.Context, req *codeActionsRequest) error
 // See [splitLines] for command implementation.
 func refactorRewriteSplitLines(ctx context.Context, req *codeActionsRequest) error {
 	// TODO(adonovan): opt: don't set needPkg just for FileSet.
-	if msg, ok, _ := canSplitLines(req.pgf.File, req.pkg.FileSet(), req.start, req.end); ok {
+	if msg, ok, _ := canSplitLines(req.pgf.Cursor, req.pkg.FileSet(), req.start, req.end); ok {
 		req.addApplyFixAction(msg, fixSplitLines, req.loc)
 	}
 	return nil
 }
 
+func refactorRewriteEliminateDotImport(ctx context.Context, req *codeActionsRequest) error {
+	// Figure out if the request is placed over a dot import.
+	var importSpec *ast.ImportSpec
+	for _, imp := range req.pgf.File.Imports {
+		if posRangeContains(imp.Pos(), imp.End(), req.start, req.end) {
+			importSpec = imp
+			break
+		}
+	}
+	if importSpec == nil {
+		return nil
+	}
+	if importSpec.Name == nil || importSpec.Name.Name != "." {
+		return nil
+	}
+
+	// dotImported package path and its imported name after removing the dot.
+	imported := req.pkg.TypesInfo().PkgNameOf(importSpec).Imported()
+	newName := imported.Name()
+
+	rng, err := req.pgf.PosRange(importSpec.Name.Pos(), importSpec.Path.Pos())
+	if err != nil {
+		return err
+	}
+	// Delete the '.' part of the import.
+	edits := []protocol.TextEdit{{
+		Range: rng,
+	}}
+
+	fileScope, ok := req.pkg.TypesInfo().Scopes[req.pgf.File]
+	if !ok {
+		return nil
+	}
+
+	// Go through each use of the dot imported package, checking its scope for
+	// shadowing and calculating an edit to qualify the identifier.
+	for curId := range req.pgf.Cursor.Preorder((*ast.Ident)(nil)) {
+		ident := curId.Node().(*ast.Ident)
+
+		// Only keep identifiers that use a symbol from the
+		// dot imported package.
+		use := req.pkg.TypesInfo().Uses[ident]
+		if use == nil || use.Pkg() == nil {
+			continue
+		}
+		if use.Pkg() != imported {
+			continue
+		}
+
+		// Only qualify unqualified identifiers (due to dot imports)
+		// that reference package-level symbols.
+		// All other references to a symbol imported from another package
+		// are nested within a select expression (pkg.Foo, v.Method, v.Field).
+		if ek, _ := curId.ParentEdge(); ek == edge.SelectorExpr_Sel {
+			continue // qualified identifier (pkg.X) or selector (T.X or e.X)
+		}
+		if !typesinternal.IsPackageLevel(use) {
+			continue // unqualified field reference T{X: ...}
+		}
+
+		// Make sure that the package name will not be shadowed by something else in scope.
+		// If it is then we cannot offer this particular code action.
+		//
+		// TODO: If the object found in scope is the package imported without a
+		// dot, or some builtin not used in the file, the code action could be
+		// allowed to go through.
+		sc := fileScope.Innermost(ident.Pos())
+		if sc == nil {
+			continue
+		}
+		_, obj := sc.LookupParent(newName, ident.Pos())
+		if obj != nil {
+			continue
+		}
+
+		rng, err := req.pgf.PosRange(ident.Pos(), ident.Pos()) // sic, zero-width range before ident
+		if err != nil {
+			continue
+		}
+		edits = append(edits, protocol.TextEdit{
+			Range:   rng,
+			NewText: newName + ".",
+		})
+	}
+
+	req.addEditAction("Eliminate dot import", nil, protocol.DocumentChangeEdit(
+		req.fh,
+		edits,
+	))
+	return nil
+}
+
 // refactorRewriteJoinLines produces "Join ITEMS into one line" code actions.
 // See [joinLines] for command implementation.
 func refactorRewriteJoinLines(ctx context.Context, req *codeActionsRequest) error {
 	// TODO(adonovan): opt: don't set needPkg just for FileSet.
-	if msg, ok, _ := canJoinLines(req.pgf.File, req.pkg.FileSet(), req.start, req.end); ok {
+	if msg, ok, _ := canJoinLines(req.pgf.Cursor, req.pkg.FileSet(), req.start, req.end); ok {
 		req.addApplyFixAction(msg, fixJoinLines, req.loc)
 	}
 	return nil
@@ -721,6 +835,82 @@ func refactorRewriteFillSwitch(ctx context.Context, req *codeActionsRequest) err
 	return nil
 }
 
+// selectionContainsStructField returns true if the given struct contains a
+// field between start and end pos. If needsTag is true, it only returns true if
+// the struct field found contains a struct tag.
+func selectionContainsStructField(node *ast.StructType, start, end token.Pos, needsTag bool) bool {
+	for _, field := range node.Fields.List {
+		if start <= field.End() && end >= field.Pos() {
+			if !needsTag || field.Tag != nil {
+				return true
+			}
+		}
+	}
+	return false
+}
+
+// selectionContainsStruct returns true if there exists a struct containing
+// fields within start and end positions. If removeTags is true, it means the
+// current command is for remove tags rather than add tags, so we only return
+// true if the struct field found contains a struct tag to remove.
+func selectionContainsStruct(cursor inspector.Cursor, start, end token.Pos, removeTags bool) bool {
+	cur, ok := cursor.FindByPos(start, end)
+	if !ok {
+		return false
+	}
+	if _, ok := cur.Node().(*ast.StructType); ok {
+		return true
+	}
+
+	// Handles case where selection is within struct.
+	for c := range cur.Enclosing((*ast.StructType)(nil)) {
+		if selectionContainsStructField(c.Node().(*ast.StructType), start, end, removeTags) {
+			return true
+		}
+	}
+
+	// Handles case where selection contains struct but may contain other nodes, including other structs.
+	for c := range cur.Preorder((*ast.StructType)(nil)) {
+		node := c.Node().(*ast.StructType)
+		// Check that at least one field is located within the selection. If we are removing tags, that field
+		// must also have a struct tag, otherwise we do not provide the code action.
+		if selectionContainsStructField(node, start, end, removeTags) {
+			return true
+		}
+	}
+	return false
+}
+
+// refactorRewriteAddStructTags produces "Add struct tags" code actions.
+// See [server.commandHandler.ModifyTags] for command implementation.
+func refactorRewriteAddStructTags(ctx context.Context, req *codeActionsRequest) error {
+	if selectionContainsStruct(req.pgf.Cursor, req.start, req.end, false) {
+		// TODO(mkalil): Prompt user for modification args once we have dialogue capabilities.
+		cmdAdd := command.NewModifyTagsCommand("Add struct tags", command.ModifyTagsArgs{
+			URI:   req.loc.URI,
+			Range: req.loc.Range,
+			Add:   "json",
+		})
+		req.addCommandAction(cmdAdd, false)
+	}
+	return nil
+}
+
+// refactorRewriteRemoveStructTags produces "Remove struct tags" code actions.
+// See [server.commandHandler.ModifyTags] for command implementation.
+func refactorRewriteRemoveStructTags(ctx context.Context, req *codeActionsRequest) error {
+	// TODO(mkalil): Prompt user for modification args once we have dialogue capabilities.
+	if selectionContainsStruct(req.pgf.Cursor, req.start, req.end, true) {
+		cmdRemove := command.NewModifyTagsCommand("Remove struct tags", command.ModifyTagsArgs{
+			URI:   req.loc.URI,
+			Range: req.loc.Range,
+			Clear: true,
+		})
+		req.addCommandAction(cmdRemove, false)
+	}
+	return nil
+}
+
 // removableParameter returns paramInfo about a removable parameter indicated
 // by the given [start, end) range, or nil if no such removal is available.
 //
@@ -789,6 +979,17 @@ func refactorInlineCall(ctx context.Context, req *codeActionsRequest) error {
 	return nil
 }
 
+// refactorInlineVariable produces the "Inline variable 'v'" code action.
+// See [inlineVariableOne] for command implementation.
+func refactorInlineVariable(ctx context.Context, req *codeActionsRequest) error {
+	// TODO(adonovan): offer "inline all" variant that eliminates the var (see #70085).
+	if curUse, _, ok := canInlineVariable(req.pkg.TypesInfo(), req.pgf.Cursor, req.start, req.end); ok {
+		title := fmt.Sprintf("Inline variable %q", curUse.Node().(*ast.Ident).Name)
+		req.addApplyFixAction(title, fixInlineVariable, req.loc)
+	}
+	return nil
+}
+
 // goTest produces "Run tests and benchmarks" code actions.
 // See [server.commandHandler.runTests] for command implementation.
 func goTest(ctx context.Context, req *codeActionsRequest) error {
@@ -847,43 +1048,65 @@ func goAssembly(ctx context.Context, req *codeActionsRequest) error {
 	// directly to (say) a lambda of interest.
 	// Perhaps we could scroll to STEXT for the innermost
 	// enclosing nested function?
-	path, _ := astutil.PathEnclosingInterval(req.pgf.File, req.start, req.end)
-	if len(path) >= 2 { // [... FuncDecl File]
-		if decl, ok := path[len(path)-2].(*ast.FuncDecl); ok {
-			if fn, ok := req.pkg.TypesInfo().Defs[decl.Name].(*types.Func); ok {
-				sig := fn.Signature()
 
-				// Compute the linker symbol of the enclosing function.
-				var sym strings.Builder
-				if fn.Pkg().Name() == "main" {
-					sym.WriteString("main")
-				} else {
-					sym.WriteString(fn.Pkg().Path())
-				}
-				sym.WriteString(".")
-				if sig.Recv() != nil {
-					if isPtr, named := typesinternal.ReceiverNamed(sig.Recv()); named != nil {
-						if isPtr {
-							fmt.Fprintf(&sym, "(*%s)", named.Obj().Name())
-						} else {
-							sym.WriteString(named.Obj().Name())
+	// Compute the linker symbol of the enclosing function or var initializer.
+	var sym strings.Builder
+	if pkg := req.pkg.Types(); pkg.Name() == "main" {
+		sym.WriteString("main")
+	} else {
+		sym.WriteString(pkg.Path())
+	}
+	sym.WriteString(".")
+
+	curSel, _ := req.pgf.Cursor.FindByPos(req.start, req.end)
+	for cur := range curSel.Enclosing((*ast.FuncDecl)(nil), (*ast.ValueSpec)(nil)) {
+		var name string // in command title
+		switch node := cur.Node().(type) {
+		case *ast.FuncDecl:
+			// package-level func or method
+			if fn, ok := req.pkg.TypesInfo().Defs[node.Name].(*types.Func); ok &&
+				fn.Name() != "_" { // blank functions are not compiled
+
+				// Source-level init functions are compiled (along with
+				// package-level var initializers) in into a single pkg.init
+				// function, so this falls out of the logic below.
+
+				if sig := fn.Signature(); sig.TypeParams() == nil && sig.RecvTypeParams() == nil { // generic => no assembly
+					if sig.Recv() != nil {
+						if isPtr, named := typesinternal.ReceiverNamed(sig.Recv()); named != nil {
+							if isPtr {
+								fmt.Fprintf(&sym, "(*%s)", named.Obj().Name())
+							} else {
+								sym.WriteString(named.Obj().Name())
+							}
+							sym.WriteByte('.')
 						}
-						sym.WriteByte('.')
 					}
-				}
-				sym.WriteString(fn.Name())
+					sym.WriteString(fn.Name())
 
-				if fn.Name() != "_" && // blank functions are not compiled
-					(fn.Name() != "init" || sig.Recv() != nil) && // init functions aren't linker functions
-					sig.TypeParams() == nil && sig.RecvTypeParams() == nil { // generic => no assembly
-					cmd := command.NewAssemblyCommand(
-						fmt.Sprintf("Browse %s assembly for %s", view.GOARCH(), decl.Name),
-						view.ID(),
-						string(req.pkg.Metadata().ID),
-						sym.String())
-					req.addCommandAction(cmd, false)
+					name = node.Name.Name // success
 				}
 			}
+
+		case *ast.ValueSpec:
+			// package-level var initializer?
+			if len(node.Names) > 0 && len(node.Values) > 0 {
+				v := req.pkg.TypesInfo().Defs[node.Names[0]]
+				if v != nil && typesinternal.IsPackageLevel(v) {
+					sym.WriteString("init")
+					name = "package initializer" // success
+				}
+			}
+		}
+
+		if name != "" {
+			cmd := command.NewAssemblyCommand(
+				fmt.Sprintf("Browse %s assembly for %s", view.GOARCH(), name),
+				view.ID(),
+				string(req.pkg.Metadata().ID),
+				sym.String())
+			req.addCommandAction(cmd, false)
+			break
 		}
 	}
 	return nil
@@ -894,7 +1117,7 @@ func goAssembly(ctx context.Context, req *codeActionsRequest) error {
 func toggleCompilerOptDetails(ctx context.Context, req *codeActionsRequest) error {
 	// TODO(adonovan): errors from code action providers should probably be
 	// logged, even if they aren't visible to the client; see https://go.dev/issue/71275.
-	if meta, err := NarrowestMetadataForFile(ctx, req.snapshot, req.fh.URI()); err == nil {
+	if meta, err := req.snapshot.NarrowestMetadataForFile(ctx, req.fh.URI()); err == nil {
 		if len(meta.CompiledGoFiles) == 0 {
 			return fmt.Errorf("package %q does not compile file %q", meta.ID, req.fh.URI())
 		}
@@ -902,7 +1125,7 @@ func toggleCompilerOptDetails(ctx context.Context, req *codeActionsRequest) erro
 
 		title := fmt.Sprintf("%s compiler optimization details for %q",
 			cond(req.snapshot.WantCompilerOptDetails(dir), "Hide", "Show"),
-			filepath.Base(dir.Path()))
+			dir.Base())
 		cmd := command.NewGCDetailsCommand(title, req.fh.URI())
 		req.addCommandAction(cmd, false)
 	}
