@@ -14,7 +14,7 @@ package golang
 // - rewrite using html/template.
 //   Or factor with golang.org/x/pkgsite/internal/godoc/dochtml.
 // - emit breadcrumbs for parent + sibling packages.
-// - list promoted methods---we have type information!
+// - list promoted methods---we have type information! (golang/go#67158)
 // - gather Example tests, following go/doc and pkgsite.
 // - add option for doc.AllDecls: show non-exported symbols too.
 // - style the <li> bullets in the index as invisible.
@@ -39,7 +39,6 @@ import (
 	"go/token"
 	"go/types"
 	"html"
-	"iter"
 	"path/filepath"
 	"slices"
 	"strings"
@@ -50,7 +49,6 @@ import (
 	"golang.org/x/tools/gopls/internal/protocol"
 	goplsastutil "golang.org/x/tools/gopls/internal/util/astutil"
 	"golang.org/x/tools/gopls/internal/util/bug"
-	"golang.org/x/tools/gopls/internal/util/safetoken"
 	"golang.org/x/tools/internal/stdlib"
 	"golang.org/x/tools/internal/typesinternal"
 )
@@ -329,7 +327,17 @@ func PackageDocHTML(viewID string, pkg *cache.Package, web Web) ([]byte, error)
 			filterValues(&t.Vars)
 			filterFuncs(&t.Funcs)
 			filterFuncs(&t.Methods)
-			return unexported(t.Name)
+			if unexported(t.Name) {
+				// If an unexported type has an exported constructor function,
+				// treat the constructor as an ordinary standalone function.
+				// We will sort Funcs again below.
+				docpkg.Funcs = append(docpkg.Funcs, t.Funcs...)
+				return true // delete this type
+			}
+			return false // keep this type
+		})
+		slices.SortFunc(docpkg.Funcs, func(x, y *doc.Func) int {
+			return strings.Compare(x.Name, y.Name)
 		})
 	}
 
@@ -568,15 +576,12 @@ window.addEventListener('load', function() {
 					if !to.IsValid() {
 						bug.Reportf("invalid Pos")
 					}
-					start, err := safetoken.Offset(file.Tok, pos)
+					text, err := file.PosText(pos, to)
 					if err != nil {
-						bug.Reportf("invalid start Pos: %v", err)
+						bug.Reportf("invalid pos range: %v", err)
+						return
 					}
-					end, err := safetoken.Offset(file.Tok, to)
-					if err != nil {
-						bug.Reportf("invalid end Pos: %v", err)
-					}
-					buf.WriteString(escape(string(file.Src[start:end])))
+					buf.WriteString(escape(string(text)))
 					pos = to
 				}
 				ast.Inspect(n, func(n ast.Node) bool {
@@ -666,7 +671,7 @@ window.addEventListener('load', function() {
 				cloneTparams(sig.RecvTypeParams()),
 				cloneTparams(sig.TypeParams()),
 				types.NewTuple(append(
-					slices.Collect(tupleVariables(sig.Params()))[:3],
+					slices.Collect(sig.Params().Variables())[:3],
 					types.NewParam(0, nil, "", types.Typ[types.Invalid]))...),
 				sig.Results(),
 				false) // any final ...T parameter is truncated
@@ -851,17 +856,3 @@ window.addEventListener('load', function() {
 
 	return buf.Bytes(), nil
 }
-
-// tupleVariables returns a go1.23 iterator over the variables of a tuple type.
-//
-// Example: for v := range tuple.Variables() { ... }
-// TODO(adonovan): use t.Variables in go1.24.
-func tupleVariables(t *types.Tuple) iter.Seq[*types.Var] {
-	return func(yield func(v *types.Var) bool) {
-		for i := range t.Len() {
-			if !yield(t.At(i)) {
-				break
-			}
-		}
-	}
-}
