@@ -38,13 +38,13 @@ import (
 	"golang.org/x/tools/gopls/internal/protocol/command"
 	"golang.org/x/tools/gopls/internal/settings"
 	"golang.org/x/tools/gopls/internal/util/bug"
+	"golang.org/x/tools/gopls/internal/util/tokeninternal"
 	"golang.org/x/tools/gopls/internal/vulncheck"
 	"golang.org/x/tools/gopls/internal/vulncheck/scan"
 	"golang.org/x/tools/internal/diff"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/gocommand"
 	"golang.org/x/tools/internal/jsonrpc2"
-	"golang.org/x/tools/internal/tokeninternal"
 	"golang.org/x/tools/internal/xcontext"
 )
 
@@ -1604,6 +1604,7 @@ func showDocumentImpl(ctx context.Context, cli protocol.Client, url protocol.URI
 }
 
 func (c *commandHandler) ChangeSignature(ctx context.Context, args command.ChangeSignatureArgs) (*protocol.WorkspaceEdit, error) {
+	countChangeSignature.Inc()
 	var result *protocol.WorkspaceEdit
 	err := c.run(ctx, commandConfig{
 		forURI: args.Location.URI,
@@ -1704,6 +1705,16 @@ func (c *commandHandler) FreeSymbols(ctx context.Context, viewID string, loc pro
 	return nil
 }
 
+func (c *commandHandler) SplitPackage(ctx context.Context, viewID, packageID string) error {
+	web, err := c.s.getWeb()
+	if err != nil {
+		return err
+	}
+	url := web.splitpkgURL(viewID, packageID)
+	openClientBrowser(ctx, c.s.client, "SplitPackage", url, c.s.Options())
+	return nil
+}
+
 func (c *commandHandler) Assembly(ctx context.Context, viewID, packageID, symbol string) error {
 	web, err := c.s.getWeb()
 	if err != nil {
