@@ -70,7 +70,7 @@ func (s *server) Diagnostic(ctx context.Context, params *protocol.DocumentDiagno
 	return &protocol.DocumentDiagnosticReport{
 		Value: protocol.RelatedFullDocumentDiagnosticReport{
 			FullDocumentDiagnosticReport: protocol.FullDocumentDiagnosticReport{
-				Items: toProtocolDiagnostics(diagnostics),
+				Items: cache.ToProtocolDiagnostics(diagnostics...),
 			},
 		},
 	}, nil
@@ -212,7 +212,7 @@ func (s *server) diagnoseSnapshot(ctx context.Context, snapshot *cache.Snapshot,
 		// The second phase runs after the delay, and does everything.
 
 		if len(changedURIs) > 0 {
-			diagnostics, err := s.diagnoseChangedFiles(ctx, snapshot, changedURIs)
+			diagnostics, err := diagnoseChangedFiles(ctx, snapshot, changedURIs)
 			if err != nil {
 				if ctx.Err() == nil {
 					event.Error(ctx, "warning: while diagnosing changed files", err, snapshot.Labels()...)
@@ -239,7 +239,7 @@ func (s *server) diagnoseSnapshot(ctx context.Context, snapshot *cache.Snapshot,
 	s.updateDiagnostics(ctx, snapshot, diagnostics, true)
 }
 
-func (s *server) diagnoseChangedFiles(ctx context.Context, snapshot *cache.Snapshot, uris []protocol.DocumentURI) (diagMap, error) {
+func diagnoseChangedFiles(ctx context.Context, snapshot *cache.Snapshot, uris []protocol.DocumentURI) (diagMap, error) {
 	ctx, done := event.Start(ctx, "server.diagnoseChangedFiles", snapshot.Labels()...)
 	defer done()
 
@@ -902,7 +902,7 @@ func (s *server) publishFileDiagnosticsLocked(ctx context.Context, views viewSet
 	// Publish, if necessary.
 	if hash != f.publishedHash || f.mustPublish {
 		if err := s.client.PublishDiagnostics(ctx, &protocol.PublishDiagnosticsParams{
-			Diagnostics: toProtocolDiagnostics(unique),
+			Diagnostics: cache.ToProtocolDiagnostics(unique...),
 			URI:         uri,
 			Version:     version, // 0 ("on disk") => omitted from JSON encoding
 		}); err != nil {
@@ -914,33 +914,6 @@ func (s *server) publishFileDiagnosticsLocked(ctx context.Context, views viewSet
 	return nil
 }
 
-func toProtocolDiagnostics(diagnostics []*cache.Diagnostic) []protocol.Diagnostic {
-	// TODO(rfindley): support bundling edits, and bundle all suggested fixes here.
-	// (see cache.bundleLazyFixes).
-
-	reports := []protocol.Diagnostic{}
-	for _, diag := range diagnostics {
-		pdiag := protocol.Diagnostic{
-			// diag.Message might start with \n or \t
-			Message:            strings.TrimSpace(diag.Message),
-			Range:              diag.Range,
-			Severity:           diag.Severity,
-			Source:             string(diag.Source),
-			Tags:               protocol.NonNilSlice(diag.Tags),
-			RelatedInformation: diag.Related,
-			Data:               diag.BundledFixes,
-		}
-		if diag.Code != "" {
-			pdiag.Code = diag.Code
-		}
-		if diag.CodeHref != "" {
-			pdiag.CodeDescription = &protocol.CodeDescription{Href: diag.CodeHref}
-		}
-		reports = append(reports, pdiag)
-	}
-	return reports
-}
-
 func (s *server) shouldIgnoreError(snapshot *cache.Snapshot, err error) bool {
 	if err == nil { // if there is no error at all
 		return false
