@@ -39,7 +39,7 @@ func DefaultOptions(overrides ...func(*Options)) *Options {
 				DynamicWatchedFilesSupported:               true,
 				LineFoldingOnly:                            false,
 				HierarchicalDocumentSymbolSupport:          true,
-				ImportsSource:                              ImportsSourceGoimports,
+				ImportsSource:                              ImportsSourceGopls,
 			},
 			ServerOptions: ServerOptions{
 				SupportedCodeActions: map[file.Kind]map[protocol.CodeActionKind]bool{
@@ -53,6 +53,7 @@ func DefaultOptions(overrides ...func(*Options)) *Options {
 						GoAssembly:                       true,
 						GoDoc:                            true,
 						GoFreeSymbols:                    true,
+						GoSplitPackage:                   true,
 						GoplsDocFeatures:                 true,
 						RefactorRewriteChangeQuote:       true,
 						RefactorRewriteFillStruct:        true,
@@ -103,7 +104,9 @@ func DefaultOptions(overrides ...func(*Options)) *Options {
 						DiagnosticsTrigger:        DiagnosticsOnEdit,
 						AnalysisProgressReporting: true,
 					},
-					InlayHintOptions: InlayHintOptions{},
+					InlayHintOptions: InlayHintOptions{
+						Hints: map[InlayHint]bool{},
+					},
 					DocumentationOptions: DocumentationOptions{
 						HoverKind:    FullDocumentation,
 						LinkTarget:   "pkg.go.dev",
@@ -150,5 +153,6 @@ func DefaultOptions(overrides ...func(*Options)) *Options {
 			override(options)
 		}
 	}
+
 	return options
 }
