@@ -394,15 +394,18 @@ func F[T comparable]() {}
 	Run(t, mod, func(t *testing.T, env *Env) {
 		env.OpenFile("main.go")
 
-		// TypeDefinition of comparable should
-		// returns an empty result, not panic.
+		// TypeDefinition of comparable should not panic.
 		loc := env.RegexpSearch("main.go", "comparable")
-		locs, err := env.Editor.TypeDefinitions(env.Ctx, loc)
+		locs, err := env.Editor.TypeDefinitions(env.Ctx, loc) // doesn't panic
 		if err != nil {
 			t.Fatal(err)
 		}
-		if len(locs) > 0 {
-			t.Fatalf("unexpected result: %v", locs)
+
+		// For extra credit, check the actual location.
+		got := fmt.Sprint(locs)
+		wantSubstr := "builtin.go"
+		if !strings.Contains(got, wantSubstr) {
+			t.Errorf("TypeDefinitions('comparable') = %v, want substring %q", got, wantSubstr)
 		}
 	})
 }
@@ -454,7 +457,7 @@ func TestGoToCrashingDefinition_Issue49223(t *testing.T) {
 		params.TextDocument.URI = protocol.DocumentURI("fugitive%3A///Users/user/src/mm/ems/.git//0/pkg/domain/treasury/provider.go")
 		params.Position.Character = 18
 		params.Position.Line = 0
-		env.Editor.Server.Definition(env.Ctx, params)
+		env.Editor.Server.Definition(env.Ctx, params) // ignore error
 	})
 }
 
@@ -699,3 +702,51 @@ func Foo() {
 		}
 	})
 }
+
+func TestCommentDefinition_Issue69616(t *testing.T) {
+	// This test exercises a few edge cases discovered by telemetry in
+	// golang/go#69616, namely situations where a parsed Go file might
+	// not have an associated scope in the package types.Info.
+	//
+	// The files below set up two types of edge cases:
+	//  - a 'compiled' Go file that isn't actually type-checked, because it has
+	//    the wrong package name
+	//  - a non-compiled Go file (unsafe.go).
+	const src = `
+-- go.mod --
+module mod.com
+
+go 1.21
+-- cmd/main.go --
+package main
+
+import "unsafe"
+
+var _  = unsafe.Offsetof
+
+func Foo() {}
+-- cmd/x.go --
+package x
+
+// Bar is like [Foo]
+func Bar() {}
+`
+
+	Run(t, src, func(t *testing.T, env *Env) {
+		// First, check that we don't produce a crash or bug when
+		// finding definitions in a 'compiled' go file that isn't actually type
+		// checked.
+		env.OpenFile("cmd/x.go")
+		_, _ = env.Editor.Definitions(env.Ctx, env.RegexpSearch("cmd/x.go", "()Foo"))
+
+		// Next, go to the unsafe package, and find the doc link to [Sizeof].
+		// It will also fail to resolve, because unsafe.go isn't compiled, but
+		// again should not panic or result in a bug.
+		env.OpenFile("cmd/main.go")
+		loc := env.FirstDefinition(env.RegexpSearch("cmd/main.go", `unsafe\.(Offsetof)`))
+		unsafePath := loc.URI.Path()
+		env.OpenFile(unsafePath)
+		_, _ = env.Editor.Definitions(env.Ctx,
+			env.RegexpSearch(unsafePath, `\[()Sizeof\]`))
+	})
+}
