@@ -194,7 +194,23 @@ func Test(t *testing.T) {
 				diags:      make(map[protocol.Location][]protocol.Diagnostic),
 				extraNotes: make(map[protocol.DocumentURI]map[string][]*expect.Note),
 			}
-			defer run.env.Shutdown()
+			defer run.env.Close()
+
+			// Support built-in pseudo-locations here.
+			// fmtLoc coerces "got" Locations to these forms
+			// so they can be compared without depending
+			// on line numbers in external files.
+			//
+			// (We could actually define "builtin:int" etc
+			// with the correct file/line/col for each
+			// builtin symbol, but that doesn't seem to be
+			// where the bugs are.)
+			run.values["BUILTIN"] = protocol.Location{
+				URI: "file:///pseudo/builtin/builtin.go",
+			}
+			run.values["UNSAFE"] = protocol.Location{
+				URI: "file:///pseudo/unsafe/unsafe.go",
+			}
 
 			// Open all files so that we operate consistently with LSP clients, and
 			// (pragmatically) so that we have a Mapper available via the fake
@@ -612,7 +628,7 @@ var actionMarkerFuncs = map[string]func(marker){
 	"token":            actionMarkerFunc(tokenMarker),
 	"typedef":          actionMarkerFunc(typedefMarker),
 	"workspacesymbol":  actionMarkerFunc(workspaceSymbolMarker),
-	"mcptool":          actionMarkerFunc(mcpToolMarker, "output"),
+	"mcptool":          actionMarkerFunc(mcpToolMarker, "location", "output"),
 }
 
 // markerTest holds all the test data extracted from a test txtar archive.
@@ -765,7 +781,7 @@ func loadMarkerTests(dir string) ([]*markerTest, error) {
 				return err
 			}
 
-			name := strings.TrimPrefix(path, dir+string(filepath.Separator))
+			name := filepath.ToSlash(strings.TrimPrefix(path, dir+string(filepath.Separator)))
 			test, err := loadMarkerTest(name, content)
 			if err != nil {
 				return fmt.Errorf("%s: %v", path, err)
@@ -987,15 +1003,13 @@ func newEnv(t *testing.T, cache *cache.Cache, files, proxyFiles map[string][]byt
 
 	awaiter := integration.NewAwaiter(sandbox.Workdir)
 
-	var eventChan chan lsprpc.SessionEvent
+	ss := lsprpc.NewStreamServer(cache, false, nil)
+
 	var mcpServer *httptest.Server
 	if enableMCP {
-		eventChan = make(chan lsprpc.SessionEvent)
-		mcpServer = httptest.NewServer(internalmcp.HTTPHandler(eventChan, false))
+		mcpServer = httptest.NewServer(internalmcp.HTTPHandler(ss, false))
 	}
 
-	ss := lsprpc.NewStreamServer(cache, false, eventChan, nil)
-
 	server := servertest.NewPipeServer(ss, jsonrpc2.NewRawStream)
 	editor, err := fake.NewEditor(sandbox, config).Connect(ctx, server, awaiter.Hooks())
 	if err != nil {
@@ -1014,7 +1028,7 @@ func newEnv(t *testing.T, cache *cache.Cache, files, proxyFiles map[string][]byt
 	var mcpSession *mcp.ClientSession
 	if enableMCP {
 		client := mcp.NewClient("test", "v1.0.0", nil)
-		mcpSession, err = client.Connect(ctx, mcp.NewSSEClientTransport(mcpServer.URL))
+		mcpSession, err = client.Connect(ctx, mcp.NewSSEClientTransport(mcpServer.URL, nil))
 		if err != nil {
 			t.Fatalf("fail to connect to mcp server: %v", err)
 		}
@@ -1028,7 +1042,6 @@ func newEnv(t *testing.T, cache *cache.Cache, files, proxyFiles map[string][]byt
 		Awaiter:    awaiter,
 		MCPSession: mcpSession,
 		MCPServer:  mcpServer,
-		EventChan:  eventChan,
 	}
 }
 
@@ -1100,6 +1113,16 @@ func (run *markerTestRun) fmtLoc(loc protocol.Location) string {
 		run.env.TB.Errorf("unable to find %s in test archive", loc)
 		return "<invalid location>"
 	}
+
+	// Format builtin locations as "file:///pseudo/builtin/builtin.go:0:0",
+	// which is the predefined BUILTIN marker; ditto UNSAFE.
+	if strings.HasSuffix(string(loc.URI), "src/unsafe/unsafe.go") ||
+		strings.HasSuffix(string(loc.URI), "src/builtin/builtin.go") {
+		name := filepath.Base(filepath.Dir(loc.URI.Path())) // "unsafe" or "builtin"
+		loc.URI = protocol.DocumentURI("file:///pseudo/" + name + "/" + name + ".go")
+		loc.Range = protocol.Range{}
+	}
+
 	lines := bytes.Count(run.test.archive.Comment, []byte("\n"))
 	var name string
 	for _, f := range run.test.archive.Files {
@@ -1253,9 +1276,8 @@ func convertNamedArgLocation(mark marker, arg any) (protocol.Location, error) {
 		if v, ok := mark.run.values[id]; ok {
 			if loc, ok := v.(protocol.Location); ok {
 				return loc, nil
-			} else {
-				return protocol.Location{}, fmt.Errorf("invalid location value %v", v)
 			}
+			return protocol.Location{}, fmt.Errorf("invalid location value %v", v)
 		}
 	}
 	return convertLocation(mark, arg)
@@ -2022,7 +2044,7 @@ func rename(env *integration.Env, loc protocol.Location, newName string) (map[st
 // See also:
 //   - Editor.applyWorkspaceEdit ../integration/fake/editor.go for the
 //     implementation of this operation used in normal testing.
-//   - cmdClient.applyWorkspaceEdit in ../../../cmd/cmd.go for the
+//   - client.applyWorkspaceEdit in ../../../cmd/cmd.go for the
 //     CLI variant.
 func changedFiles(env *integration.Env, changes []protocol.DocumentChange) (map[string][]byte, error) {
 	uriToPath := env.Sandbox.Workdir.URIToPath
@@ -2439,15 +2461,43 @@ func implementationMarker(mark marker, src protocol.Location, want ...protocol.L
 	}
 }
 
-func mcpToolMarker(mark marker, tool string, rawArgs string, loc protocol.Location) {
+func mcpToolMarker(mark marker, tool string, rawArgs string) {
+	if !mark.run.test.mcp {
+		mark.errorf("mcp not enabled: add -mcp")
+		return
+	}
 	args := make(map[string]any)
 	if err := json.Unmarshal([]byte(rawArgs), &args); err != nil {
 		mark.errorf("fail to unmarshal arguments to map[string]any: %v", err)
 		return
 	}
 
-	// TODO(hxjiang): Make the "location" key configurable.
-	args["location"] = loc
+	// substitutePaths replaces instances of $WORKDIR in string or []string values with
+	// the actual working directory.
+	var substitutePaths func(any) any
+	substitutePaths = func(v any) any {
+		switch v := v.(type) {
+		case string:
+			return strings.ReplaceAll(v, "$WORKDIR", mark.run.env.Sandbox.Workdir.RootURI().Path())
+
+		case []any:
+			for i, e := range v {
+				if _, ok := e.(string); ok {
+					v[i] = substitutePaths(e).(string)
+				}
+			}
+		}
+		return v
+	}
+	// Hack: replace '$WORKDIR' in string arguments with the actual sandbox
+	// working directory.
+	for k, v := range args {
+		args[k] = substitutePaths(v)
+	}
+
+	if loc := namedArg(mark, "location", protocol.Location{}); loc != (protocol.Location{}) {
+		args["location"] = loc
+	}
 
 	res, err := mark.run.env.MCPSession.CallTool(mark.ctx(), &mcp.CallToolParams{
 		Name:      tool,
@@ -2471,13 +2521,18 @@ func mcpToolMarker(mark marker, tool string, rawArgs string, loc protocol.Locati
 	}
 
 	got := buf.String()
+	// For portability, replace all (potential) filepath separators with "/".
+	got = strings.ReplaceAll(got, string(filepath.Separator), "/")
+	// To ensure consistent unified diff output, the working directory path
+	// is replaced with "$WORKDIR". This addresses cases where MCP tools
+	// include absolute file paths in generated diffs.
+	got = strings.ReplaceAll(got, filepath.ToSlash(mark.run.env.Sandbox.Workdir.RootURI().Path()), "$WORKDIR")
 
 	output := namedArg(mark, "output", expect.Identifier(""))
 	golden := mark.getGolden(output)
-	if want, ok := golden.Get(mark.T(), "", []byte(got)); !ok {
-		mark.errorf("%s: missing golden file @%s", mark.note.Name, golden.id)
-	} else if diff := cmp.Diff(got, string(want)); diff != "" {
-		mark.errorf("unexpected mcp tools call %s return; got:\n%s\n want:\n%s\ndiff:\n%s", tool, got, want, diff)
+	want, _ := golden.Get(mark.T(), "", []byte(got))
+	if diff := compare.Text(string(want), got); diff != "" {
+		mark.errorf("unexpected mcp tools call %s return: diff:\n%s", tool, diff)
 	}
 }
 
